#include "ModuleBase.h"
//#include <boost/algorithm/string.hpp>

namespace mdd {
	class ModuleMerge : public ModuleBase {
	private:

	public:
		ModuleMerge();
		int addModuleInput();
		int removeModuleInput();
		std::string setInputName(const std::string& input_id, const std::string& new_name);
		bool configure(const std::string& config) override;
		state update() override;
	};
	ModuleMerge::ModuleMerge()
		:ModuleBase(R"JSON(
        {
            "inputs":{
				"value":2
			}
        })JSON")
	{
		std::vector<double> default_val = { 1 };
		inputs.push_back(std::make_shared<Input>(this, "Value", 0, default_val));
		inputs.push_back(std::make_shared<Input>(this, "Value", 1, default_val));
		outputs.push_back(std::make_shared<Output>(this, "Value", 0, default_val));
		key = "ModuleMerge";
		setName("Merge");
	}

	int ModuleMerge::addModuleInput() {
		std::vector<double> default_val = { 1 };
		inputs.push_back(std::make_shared<Input>(this, "Value", inputs.size(), default_val));
		return inputs.size();
	}

	int ModuleMerge::removeModuleInput() {
		inputs.pop_back();
		return inputs.size();
	}

	std::string ModuleMerge::setInputName(const std::string& input_id, const std::string& new_name) {
		for (int id = 0; id < inputs.size(); ++id) {
			if (getInput(id)->getID() == input_id) {
				getInput(id)->setName(new_name);
				return getInput(id)->getID();
			}
		}
		return "";
	}

	bool ModuleMerge::configure(const std::string& config) {
		bool success = false;
		success = ModuleBase::configure(config);
		json config_parsed = json::parse(config);
		json base_config = json::parse(ModuleBase::getConfiguration());
		if (config_parsed.contains("configure"))
		{
			config_parsed = config_parsed["configure"];
		}
		if (config_parsed.contains("inputs"))
		{
			json jval = config_parsed["inputs"]["value"];
			if (jval.is_string())
			{
				jval = json::parse(jval.get<std::string>());
			}
			size_t new_length = jval.get<size_t>();
			size_t length = inputs.size();
			if ((int)length - (int)new_length > 0)
			{
				inputs.erase(inputs.end() - (length - new_length), inputs.end());
			}
			else {
				for (size_t j = length; j < new_length; j++)
				{
					inputs.push_back(std::make_shared<Input>(this, "Value", j, std::vector<double> {1}));
				}
			}
			base_config["configure"]["inputs"]["value"] = jval;
		}
		_base_config =  base_config.dump();
		return true;
	}

	state ModuleMerge::update() {
		std::vector<double>  ret;
		for (size_t i = 0; i < inputs.size(); i++)
		{
			for (size_t j = 0; j < getInput(i)->getValue().size(); j++)
			{
				ret.push_back(getInput(i)->getValue()[j]);
			}
		}
		return getOutput(0)->setValue(ret);
	}
}
