#include "ModuleBase.h"
#include <iostream>

namespace mdd{
    class ModuleSwitch : public ModuleBase {
    public:
        ModuleSwitch();
        bool configure(const std::string& config) override;
        std::string getConfiguration() override;
        state update() override;
    };
    ModuleSwitch::ModuleSwitch()
        :ModuleBase(R"JSON(
        {
            "size":{
                "value": 1
            }
        })JSON")
    {    
        std::vector<double> default_val = { 1 };
        inputs.push_back(std::make_shared<Input>(this, "Switch", 0, default_val));
        inputs.push_back(std::make_shared<Input>(this, "Default", 0, default_val));
        outputs.push_back(std::make_shared<Output>(this, "Value", 0, default_val));
        key = "ModuleSwitch";
        setName("Switch");
    }
    bool ModuleSwitch::configure(const std::string& config) {
        bool success = false;
        success = ModuleBase::configure(config);
        json config_parsed = json::parse(config);
        json base_config = json::parse(ModuleBase::getConfiguration());
        if (config_parsed.contains("configure"))
        {
            config_parsed = config_parsed["configure"];
        }
        if (config_parsed.contains("size"))
        {
            json jval = config_parsed["size"]["value"];
            if (jval.is_string())
            {
                jval = json::parse(jval.get<std::string>());
            }
            size_t length = jval.get<size_t>();
            if (length != inputs.size() - 1)
            {
                size_t length_before = inputs.size();
                if (length_before != 1)
                {
                    inputs.back()->setName("Value");
                }

                for (size_t j = length_before - 1; j < length; j++)
                {
                    inputs.push_back(std::make_shared<Input>(this, "Value", inputs.size() - 1, std::vector<double>{1}));
                }
                for (size_t j = length; j < length_before - 1; j++)
                {
                    inputs.pop_back();
                }
                if (length != 0)
                {
                    inputs.back()->setName("Default");
                }
            }
            base_config["configure"]["size"]["value"] = jval;
        }
        _base_config = base_config.dump();
        return true;
    }

    std::string ModuleSwitch::getConfiguration() {
        json ret = json::parse(ModuleBase::getConfiguration());
        for (size_t i = 0; i < ret.size(); i++)
        {
            if (ret[i].contains("name"))
            {
                if (ret[i]["name"].get<std::string>() == "size")
                {
                    ret[i]["value"] = inputs.size() - 1;
                }
            }
        }
        return ret.dump();
    }

    state ModuleSwitch::update(){
        std::vector<double> ret;
        size_t length = getInput(0)->getValue().size();
        for (size_t i = 0; i < length; i++)
        {
            int index = (int)getInput(0)->getValue()[i];
           
            if (index == 0) {
                index = 1;
            }

            if (index > inputs.size() - 1) {
                index = inputs.size() - 1;
            }
            for (size_t j = 0; j < getInput(index)->getValue().size(); j++)
            {
                ret.push_back(getInput(index)->getValue()[j]);
            }
            
        }
        return getOutput(0)->setValue(ret);

    }
}